@@ -4,10 +4,12 @@
 from __future__ import annotations
 
 import enum
+import errno
+import os
 import socket as _pysocket
 import struct
 import sys
-from typing import Any, NoReturn, Optional, Tuple, Union, cast
+from typing import Any, NoReturn, Optional, Tuple, Union, cast, overload
 
 from ._tls import HandshakeStep as HandshakeStep
 from ._tls import HelloVerifyRequest
@@ -66,8 +68,9 @@ __all__ = (
     "ciphers_available",
 )
 
-# Stolen from `_socket.pyi`.
+# `_Address` stolen from `_socket.pyi`.
 _Address: TypeAlias = Union[Tuple[Any, ...], str]
+_WriteableBuffer: TypeAlias = Union[memoryview, bytearray]
 
 
 class TLSRecordHeader:
@@ -102,7 +105,7 @@ class TLSRecordHeader:
         self.length: Final = length
 
     def __str__(self) -> str:
-        return "%s(%s, %s, %s)" % (
+        return "{}({}, {}, {})".format(
             type(self).__name__,
             self.record_type,
             self.version,
@@ -110,7 +113,7 @@ class TLSRecordHeader:
         )
 
     def __repr__(self) -> str:
-        return "%s(%r, %r, %r)" % (
+        return "{}({!r}, {!r}, {!r})".format(
             type(self).__name__,
             self.record_type,
             self.version,
@@ -313,9 +316,20 @@ class TLSWrappedSocket:
         return self._buffer.read(bufsize)
 
     def recv_into(
-        self, buffer: bytes, nbytes: Optional[int] = None, flags: int = 0
-    ) -> None:
-        raise NotImplementedError
+        self,
+        buffer: _WriteableBuffer,
+        nbytes: Optional[int] = None,
+        flags: int = 0,
+    ) -> int:
+        if nbytes is None:
+            bufsize = len(buffer)
+        else:
+            bufsize = min(len(buffer), nbytes)
+        encrypted = self._socket.recv(bufsize, flags)
+        if not encrypted:
+            return 0
+        self._buffer.receive_from_network(encrypted)
+        return self._buffer.readinto(buffer, len(encrypted))
 
     def recvfrom(self, bufsize: int, flags: int = 0) -> Tuple[bytes, _Address]:
         encrypted, addr = self._socket.recvfrom(bufsize, flags)
@@ -325,7 +339,10 @@ class TLSWrappedSocket:
         return self._buffer.read(bufsize), addr
 
     def recvfrom_into(
-        self, buffer: bytes, nbytes: Optional[int] = None, flags: int = 0
+        self,
+        buffer: _WriteableBuffer,
+        nbytes: Optional[int] = None,
+        flags: int = 0,
     ) -> Tuple[int, _Address]:
         encrypted, addr = self._socket.recvfrom(
             nbytes if nbytes is not None else len(buffer), flags
@@ -356,24 +373,15 @@ class TLSWrappedSocket:
         self._buffer.consume_outgoing(amt)
         self._socket.sendall(encrypted)
 
-    def sendto(  # type: ignore[no-untyped-def]
-        self, message: bytes, *args
-    ) -> int:
+    def sendto(self, message: bytes, *args: Any) -> int:
         if not 1 <= len(args) <= 2:
             raise TypeError(
                 "sendto() takes 2 or 3 arguments (%i given)" % (1 + len(args))
             )
-        if len(args) == 1:
-            flags, address = 0, args[0]
-        else:
-            flags, address = args
 
         amt = self._buffer.write(message)
         encrypted = self._buffer.peek_outgoing(amt)
-        if flags:
-            self._socket.sendto(encrypted, flags, address)
-        else:
-            self._socket.sendto(encrypted, address)
+        self._socket.sendto(encrypted, *args)
         self._buffer.consume_outgoing(amt)
         return len(message)
 
@@ -382,17 +390,53 @@ class TLSWrappedSocket:
         self._socket.shutdown(how)
 
     # PEP 543 adds the following methods.
-
+    @overload
     def do_handshake(self) -> None:
+        ...
+
+    @overload
+    def do_handshake(self, address: _Address) -> None:
+        ...
+
+    @overload
+    def do_handshake(self, flags: int, address: _Address) -> None:
+        ...
+
+    def do_handshake(self, *args):  # type: ignore[no-untyped-def]
+        if args and self.type is not _pysocket.SOCK_DGRAM:
+            raise OSError(errno.ENOTCONN, os.strerror(errno.ENOTCONN))
+
+        if len(args) == 0:
+            flags, address = 0, None
+        elif len(args) == 1:
+            flags, address = 0, args[0]
+        elif len(args) == 2:
+            assert isinstance(args[0], int)
+            flags, address = args
+        else:
+            raise TypeError("do_handshake() takes 0, 1, or 2 arguments")
+
         while self._handshake_state is not HandshakeStep.HANDSHAKE_OVER:
             try:
                 self._buffer.do_handshake()
             except WantReadError:
-                data = self._socket.recv(1024)
+                if address is None:
+                    data = self._socket.recv(1024, flags)
+                else:
+                    data, addr = self._socket.recvfrom(1024, flags)
+                    if addr != address:
+                        # The error may not be the clearest but we'd better
+                        # bail out in any case.
+                        raise OSError(
+                            errno.ENOTCONN, os.strerror(errno.ENOTCONN)
+                        )
                 self._buffer.receive_from_network(data)
             except WantWriteError:
                 in_transit = self._buffer.peek_outgoing(1024)
-                amt = self._socket.send(in_transit)
+                if address is None:
+                    amt = self._socket.send(in_transit, flags)
+                else:
+                    amt = self._socket.sendto(in_transit, flags, address)
                 self._buffer.consume_outgoing(amt)
 
     def unwrap(self) -> _pysocket.socket:
